// -*- coding: utf-8 -*-
// SM Tuning.
//
// Funkcje pomocnicze wykorzystywane przy przetwarzaniu/modyfikowaniu
// dokumentu. Pisane tak, by można było ten skrypt winjectować
// do głównego dokumentu

var smtdoc = {

    /////////////////////////////////////////////////////////////////
    // Ułatwiacze XPath
    /////////////////////////////////////////////////////////////////

    xpath : function(document, query) {
        return document.evaluate(query, document, null,
                                 XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
    },
    // Szukanie w bloku under a nie całym dokumencie
    xpathRelative : function(document, under, query) {
        return document.evaluate(query, under, null,
                                 XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
    },

    xpathOrdered : function(document, query) {
        return document.evaluate(query, document, null,
                                 XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    },
    xpathFirst : function(document, query) {
        return document.evaluate(query, document, null,
                                 XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
    },
    xpathAsArray : function(document, query) {
        var r = document.evaluate(query, document, null,
                                  XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
        var a = new Array(r.snapshotLength);
        for (var i = 0; i < r.snapshotLength; i++) {
            //a.push(r.snapshotItem(i));  -- tak gdyby bez rozmiaru
            a[i] = r.snapshotItem(i);
        }
        return a;
    },

    // Wymienia content elementów o identyfikatorze aId i bId
    swapContent : function(document, aId, bId) {
        var a = document.getElementById(aId);
        var b = document.getElementById(bId);
        var a_val = a.innerHTML;
        a.innerHTML = b.innerHTML;
        b.innerHTML = a_val;
    },

    /////////////////////////////////////////////////////////////////
    // Obsługa styli
    /////////////////////////////////////////////////////////////////

    // Dodanie podanej deklaracji CSS do bieżącej strony
    addStyle : function(doc, cssText) {
        var head = doc.getElementsByTagName('head')[0];
        if (!head) { return; }
        style = doc.createElement("style")
        style.type = 'text/css';
        style.innerHTML = cssText;
        head.appendChild(style);
    },

    // Dodanie referencji do podanego stylesheeta. Wykorzystanie
    // to np. 
    // smtdoc.addStyleSheet(document,
    //      'chrome://smtuning/content/stylesheet/style.css');
    addStyleSheet : function(doc, styleUrl) {
        var head = doc.getElementsByTagName('head')[0];
        if (!head) { return; }
        var objCSS = doc.createElement('link');
        objCSS.setAttribute('href', styleUrl);
        objCSS.setAttribute('type', 'text/css');
        objCSS.setAttribute('rel', 'stylesheet');
        head.appendChild(objCSS);
    },

    // Dodanie kodu javascryptowego
    addJavaScript : function(doc, scriptText) {
        var head = doc.getElementsByTagName('head')[0];
        if (!head) { return; }
        newScript = doc.createElement("script")
        newScript.type = 'text/javascript';
        newScript.innerHTML = scriptText;
        head.appendChild(newScript);
    },

    // Dodanie referencji do zewnetrznego javaskryptu
    addJavaScriptFile : function(doc, scriptUrl) {
        var head = doc.getElementsByTagName('head')[0];
        if (!head) { return; }
        var obj = doc.createElement('script');
        obj.type = "text/javascript";
        obj.setAttribute('src', scriptUrl);
        head.appendChild(obj);
    },

    // Zamiana podanego obrazka na inny
    replaceImage : function(doc, replaced_name, new_url) {
        var nodes = smtdoc.xpath(
            doc, "//img[contains(@src,'" + replaced_name + "')]");
        for (var i = 0; i < nodes.snapshotLength; i++) {
            var thisImg = nodes.snapshotItem(i);
            thisImg.setAttribute("src", new_url);
        }
    },

    /////////////////////////////////////////////////////////////////
    // Specyficzne modyfikacje dokumentów SM
    /////////////////////////////////////////////////////////////////

    // Wymiana obrazków na inne.
    // Parametry:
    //   doc - dokument (HTMLDocument, w kontekście strony po prostu document)
    //   view - kontekst javascriptu (w kontekście strony window)
    //   newUrl - url nowych obrazków (katalog, bez slasza, np.
    //                   chrome://smtuning/pieces/igor_k/40  )
    //   newSize - nowy rozmiar
    //   oldPathFrag - fragment po którym znajdziemy stare obrazki
    //           (np. images/sets albo /pieces/)
    //   newExt - nowe rozszerzenie (domyślnie '.gif')
    //
    replacePieces : function(doc, view, newUrl, newSize, oldPathFrag, newExt) {
        var all_imgs = smtdoc.xpath(doc,
                                    "//img[contains(@src,'"+oldPathFrag+"')]");
        if (! newExt) {
            newExt = '.gif';
        }
        for (var i = 0; i < all_imgs.snapshotLength; i++) {
            var thImg = all_imgs.snapshotItem(i);
            thImg.src = thImg.src.replace(
                                          /^.*\/([^\/]+)$/,
                                          newUrl + "$1")
            .replace(/\.(gif|png|jpg)/, newExt);
        }
        // To avoid unsafeWindow, one could try location hack - sth like
        //   location.href = "javascript:void(PieceImageRoot = '...')";
        // (where ... should calculate set from some existing image).
        // But I am not too concerned about Austin stealing this code...
        view.PieceImageRoot = newUrl;
        view.PieceSize = newSize;
    },

    // Skorygowanie rozmiarów w ramach boardPage
    correctBoardPageSizing : function(doc, oldSize, newSize) {
        var newPx = newSize.toString() + "px";
        if (oldSize == null) {
            oldSize = 32;
        }
        var oldPx = oldSize.toString() + "px";
        var oldRe = new RegExp(oldPx, "g");
        var sized = smtdoc.xpath(doc, "//*[contains(@style,'" + oldPx + "')]");
        for (var i = 0; i < sized.snapshotLength; i++) {
            var thisOne = sized.snapshotItem(i);
            var style = thisOne.getAttribute("style");
            var newStyle = style.replace(oldRe, newPx);
            thisOne.setAttribute("style", newStyle);
        }

        // 680x475
        var sz680 = 680;
        var sz475 = 475;
        if (newSize == 32) {
            sz680 = 680;
            sz475 = 475;
        }
        else if (newSize == 40) {
            sz680 = 720;
            sz475 = 528;
        }
        else if (newSize == 48) {
            sz680 = 900;
            sz475 = 592;
        }
        else if (newSize == 64) {
            sz680 = 1124;
            sz475 = 736;
        }
        else {
            sz680 = 680 * newSize / 32;
            sz475 = 475 * newSize / 32;
        }
        smtdoc.addStyle(doc, ".fw { width: " + sz680.toString() + "; }");
        smtdoc.addStyle(doc, ".listtab { width: " + sz680.toString() + "; }");
        smtdoc.addStyle(doc, "div.tab { height: " + sz475.toString() + "px; }");
        smtdoc.addStyle(doc, "img.drag { width: " + newSize.toString()
                          + "px; height: " + newSize.toString + "px; }");
        var halfSize = newSize / 2;
        smtdoc.addStyle(doc, "img.dragsmall { width: " + halfSize.toString()
                          + "px; height: " + halfSize.toString + "px; }");
    },

    /////////////////////////////////////////////////////////////////
    // Handlery javascriptowe
    /////////////////////////////////////////////////////////////////

    zoomIn : function() {
        var oldSize = PieceSize;
        var nextSize = smtPieceSet.nextSizeUp(oldSize);
        if (nextSize) {
            var path = smtPieceSet.piecesPath(nextSize);
            smtdoc.replacePieces(document, window, path,
                                 nextSize, '/pieces/', smtPieceSet.fileExt());
            smtdoc.correctBoardPageSizing(document, oldSize, nextSize);
        } else {
            alert("You are already using greatest available size of set "
                  + smtPieceSet.label());
        }
    },

    zoomOut : function() {
        var oldSize = PieceSize;
        var nextSize = smtPieceSet.nextSizeDown(oldSize);
        if (nextSize) {
            var path = smtPieceSet.piecesPath(nextSize);
            smtdoc.replacePieces(document, window, path,
                                 nextSize, '/pieces/', smtPieceSet.fileExt());
            smtdoc.correctBoardPageSizing(document, oldSize, nextSize);
        } else {
            alert("You are already using smallest available size of set "
                  + smtPieceSet.label());
        }
    },

    resortGames : function(sort_no) {
        if (sort_no) {
            window.location.href = '/currentgames.aspx?sort=' + sort_no;
        }
    },

    redisplayDark : function(param) {
        if (param) {
            h = window.location.href;
            h = h.replace(/&hide=[a-z]*/, "");
            if (param == "all") {
                // h jest OK
            }
            else if (param == "white") {
                h = h + "&hide=black";
            }
            else if (param == "black") {
                h = h + "&hide=white";
            }
            if (window.location.href != h) {
                window.location.href = h;
            }
        }
    },

    analysisFlip : function() {
        for (var i = 0; i < AnalysisBoard.length; i++) {
            // Obracamy planszę nr i
            AnalysisBoard[i].reverse();
            for (var j = 0; j < AnalysisBoard[i].length; j++) {
                AnalysisBoard[i][j].reverse();
            }
        }
        DrawAnalysisBoard();
        // Poprawa etykiet
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anrank8', 
                           'ContentPlaceHolder1_anrank1');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anrank7', 
                           'ContentPlaceHolder1_anrank2');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anrank6', 
                           'ContentPlaceHolder1_anrank3');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anrank5', 
                           'ContentPlaceHolder1_anrank4');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anfile1', 
                           'ContentPlaceHolder1_anfile8');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anfile2', 
                           'ContentPlaceHolder1_anfile7');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anfile3', 
                           'ContentPlaceHolder1_anfile6');
        smtdoc.swapContent(document,
                           'ContentPlaceHolder1_anfile4', 
                           'ContentPlaceHolder1_anfile5');
    },

    analysisSave : function() {
        // var analysisTab = document.getElementById("ContentPlaceHolder1_tblAnalyse");
        var rows = new Array();
        var re = /([wb])([kqrbnp])[wb]\.[a-z]+$/;
        for (var row = 8; row >= 1; row --) {
            var items = new Array();
            for (var col = 1; col <= 8; col ++) {
                var img = document.getElementById("ContentPlaceHolder1_animg" + col.toString() + row.toString());
                var here = "1";
                if (img) {
                    var src = img.getAttribute("src");
                    var m = re.exec(src);
                    if (m) {
                        here = m[2];
                        if (m[1] == "w") {
                            here = here.toUpperCase();
                        }
                    }
                }
                items.push(here);
            }
            if (Flipped) {
                items.reverse();
            }
            var s = items.join("");
            // Zamiana ciągów jedynek
            var p = s.split(/(1{2,})/);
            for (var n = 1; n < p.length; n = n + 2) {
                p[n] = p[n].length;
            }
            rows.push(p.join(""));
        }
        if (Flipped) {
            rows.reverse();
        }

        var cmtBox = document.getElementById("ContentPlaceHolder1_txtComment");
        var privRadio = smtdoc.xpathFirst(document, "//input[@type='radio' and @value='private_comment']");
        if (privRadio) {
            privRadio.checked = 1;
            privRadio.click();
        }
        if (cmtBox) {
            //cmtBox.appendChild(document.createTextNode("{POS}" + desc + "{/POS}"));
            cmtBox.value += "\n" + rows.join("/") + "\n";
        }
    },

    analysisLoad : function(fen) {
        //alert(fen);
        var fenLines = fen.split(/\//);
        var board = new Array(8);
        for (var i = 0; i < 8; i++) {
            var letters = fenLines[i].split('');
            var pieces = new Array;
            while(letters.length > 0) {
                var t = letters.shift();
                var d = parseInt(t);
                if (d) {
                    while(d > 0) {
                        pieces.push('');
                        -- d;
                    }
                } else {
                    var l = t.toLowerCase();
                    if (t == l) {
                        pieces.push('b' + l);
                    } else {
                        pieces.push('w' + l);
                    }
                }
            }
            board[7-i] = pieces;
        }

        AnaMove += 1;
        AnaPickUp = '';
        AnalysisBoard.length = AnaMove;
        AnalysisBoard[AnaMove] = board;
        DrawAnalysisBoard();
        DisplayTab(4);
    }

};
